
/*//-----------------------------------------------------------------------------
	Niue Development Environment
	Copyright 2008 T. Lansbergen, All Rights Reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted for non-commercial use only.
	
	This software is provided ``as is'' and any express or implied warranties,
	including, but not limited to, the implied warranties of merchantability and
	fitness for a particular purpose are disclaimed. In no event shall
	authors be liable for any direct, indirect, incidental, special,
	exemplary, or consequential damages (including, but not limited to,
	procurement of substitute goods or services; loss of use, data, or profits;
	or business interruption) however caused and on any theory of liability,
	whether in contract, strict liability, or tort (including negligence or
	otherwise) arising in any way out of the use of this software, even if
	advised of the possibility of such damage. 

*///-----------------------------------------------------------------------------



#ifndef SNIPPETVIEW_H
#include "snippetview.h"
#endif

snippetview::snippetview(const char *name)
	   	   : BPopUpMenu(name)
{
	SetRadioMode(false);

	
	//level 1
	AddItem(mnuSnippets = new BMenu("Insert Snippet"));
	
	//level 2
	mnuSnippets->AddItem(mnuInterfaceElements = new BMenu("Interface elements"));
	mnuSnippets->AddItem(mnuLoopsStatements = new BMenu("Loops & statements"));
	mnuSnippets->AddItem(mnuCommonCode = new BMenu("Common code"));
	mnuSnippets->AddItem(mnuUserCode = new BMenu("User defined"));

	
	//level 3
	mnuInterfaceElements->AddItem(new NSnipMenuItem("Alert", NULL, "(new BAlert(\"TITLE\", \"TEXT\", \"BUTTON1\", \"BUTTON2\", \"BUTTON3\", B_WIDTH_AS_USUAL, B_INFO_ALERT))->Go();\n"));
	mnuInterfaceElements->AddSeparatorItem();
	mnuInterfaceElements->AddItem(new NSnipMenuItem("Open panel", NULL, "BFilePanel *OpenPanel = new BFilePanel(B_OPEN_PANEL, this, 0, B_FILE_NODE, false, 0, 0, false, true);\nOpenPanel->Show();\n"));
	mnuInterfaceElements->AddItem(new NSnipMenuItem("Save panel", NULL, "BFilePanel *SavePanel = new BFilePanel(B_SAVE_PANEL, this, 0, 0, false, 0, 0, false, true);\nSavePanel->Show();\n"));
	
	mnuLoopsStatements->AddItem(new NSnipMenuItem("For", NULL, "\nfor( int i=1; i<X; i++ ) \n{\n\t//action\n}\n"));
	mnuLoopsStatements->AddItem(new NSnipMenuItem("While", NULL, "\nwhile( 'condition' ) \n{\n\t//action\n}\n"));
	mnuLoopsStatements->AddItem(new NSnipMenuItem("Do", NULL, "\ndo \n{\n\t//action\n} while ( 'condition' );\n"));
	mnuLoopsStatements->AddSeparatorItem();
	mnuLoopsStatements->AddItem(new NSnipMenuItem("If", NULL, "\nif( 'condition' ) \n{\n\t//action\n}\n"));
	mnuLoopsStatements->AddItem(new NSnipMenuItem("If/else", NULL, "\nif( 'condition' ) \n{\n\t//action\n}\nelse\n{\n\t//action\n}\n"));
	mnuLoopsStatements->AddItem(new NSnipMenuItem("If/elseif", NULL, "\nif( 'condition' ) \n{\n\t//action\n}\nelse if( 'condition' )\n{\n\t//action\n}\nelse\n{\n\t//action\n}\n"));
	mnuLoopsStatements->AddSeparatorItem();
	mnuLoopsStatements->AddItem(new NSnipMenuItem("Switch", NULL, "\nswitch( 'condition' )\n{\n\tcase 1:\n\t{\n\t\t//action\n\t}\n\tbreak;\n\n\tcase 2:\n\t{\n\t\t//action\n\t}\n\tbreak;\n\n\tdefault;\n\t{\n\t\t//action\n\t}\n}\n"));
	
	mnuCommonCode->AddItem(new NSnipMenuItem("Include local", NULL, "#include \"XXX.h\""));
	mnuCommonCode->AddItem(new NSnipMenuItem("Include global", NULL, "#include <XXX.h>"));
	mnuCommonCode->AddItem(new NSnipMenuItem("Define", NULL, "#define\tXXX\t'XXXX'"));
	
	//user snippets
	mnuUserCode->AddItem(new NSnipMenuItem("Add snippet", NULL, BString("ADD_ITEM")));
	
		//open the snippets
//		BString snippetspath;
		BPath prefspath;
		
		//find prefsfile
		find_directory(B_USER_SETTINGS_DIRECTORY, &prefspath, false);
		prefspath.Append("Niue/niue_snippets");
					
		//open file for reading
		BFile rdsnippetsfile;
		rdsnippetsfile.SetTo(prefspath.Path(), B_READ_ONLY);
		if (rdsnippetsfile.IsReadable())
		{
			
			BMessage msg('SNIP');
			msg.Unflatten(&rdsnippetsfile);
			
			if(msg.CountNames(B_STRING_TYPE) > 0)
			{
				mnuUserCode->AddItem(new NSnipMenuItem("Remove all user snippets", NULL, BString("REMOVE_ITEMS")));
				mnuUserCode->AddSeparatorItem();
							
				int i = 0;
				char		*snipname;
				uint32		type;
				int32		count;
				BString		snipcode;
				
				while(msg.GetInfo(B_STRING_TYPE, i, &snipname, &type, &count) == B_OK)
				{
					if(msg.FindString(snipname, &snipcode) == B_OK)
					{
						mnuUserCode->AddItem(new NSnipMenuItem(snipname, NULL, snipcode));
					}
					i++;
				}			
				

			}			
		}
		
		rdsnippetsfile.Unset();
}


NSnipMenuItem::NSnipMenuItem(const char *label, BMessage *message, BString code)
		: BMenuItem(label, message, 0,0)
{
	CodeToInsert = code;
}

NSnipMenuItem::~NSnipMenuItem()
{
}
