
/*//-----------------------------------------------------------------------------
	Niue Development Environment
	Copyright 2008 T. Lansbergen, All Rights Reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted for non-commercial use only.
	
	This software is provided ``as is'' and any express or implied warranties,
	including, but not limited to, the implied warranties of merchantability and
	fitness for a particular purpose are disclaimed. In no event shall
	authors be liable for any direct, indirect, incidental, special,
	exemplary, or consequential damages (including, but not limited to,
	procurement of substitute goods or services; loss of use, data, or profits;
	or business interruption) however caused and on any theory of liability,
	whether in contract, strict liability, or tort (including negligence or
	otherwise) arising in any way out of the use of this software, even if
	advised of the possibility of such damage. 

*///-----------------------------------------------------------------------------

#include "vdesignview.h"

vdesignview::vdesignview(BRect rect, char *name, uint32 resizingMode, uint32 flags)
	   	   : BView(rect, name, resizingMode, flags)
{
	//define objects
	vwMain = new BView(BRect(0,0,rect.Width()-B_V_SCROLL_BAR_WIDTH,rect.Height()-B_H_SCROLL_BAR_HEIGHT),  "vwMain", B_FOLLOW_ALL_SIDES, B_WILL_DRAW);
	scrDes = new BScrollView("scrDes", vwMain, 0, B_FOLLOW_ALL_SIDES, true, true, B_NO_BORDER);
		
	vwDes = new designview(BRect(0,0,1024,768),  "vwDes", B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);

	// set objects
	vwMain->SetViewColor(blueish);
	vwDes->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
//	scrDes->ScrollBar(B_HORIZONTAL)->SetRange(0,vwDes->Frame().Width()-rect.Width()-B_V_SCROLL_BAR_WIDTH);
	

	//add objects
	AddChild(scrDes);
	vwMain->AddChild(vwDes);

	//hack to resize our scrollers
	FrameResized(Bounds().right, Bounds().bottom);
		
}

void vdesignview::FrameResized(float width, float height)
{
	//scroll math & center view
	float x=vwDes->Bounds().Width()+B_V_SCROLL_BAR_WIDTH;
	float y=vwDes->Bounds().Height()+B_H_SCROLL_BAR_HEIGHT;

	if(width < x)
	{
		scrDes->ScrollBar(B_HORIZONTAL)->SetRange(0, x-width);
		vwDes->MoveTo(0, vwDes->Frame().top);
	}
	else
	{
		scrDes->ScrollBar(B_HORIZONTAL)->SetRange(0,0);
		vwDes->MoveTo((width - x)/2, vwDes->Frame().top);
	}

	if(height < y)
	{
		scrDes->ScrollBar(B_VERTICAL)->SetRange(0, y-height);
		vwDes->MoveTo(vwDes->Frame().left, 0);
	}
	else
	{
		scrDes->ScrollBar(B_VERTICAL)->SetRange(0,0);
		vwDes->MoveTo(vwDes->Frame().left, (height - y)/2);		
	}

		
}

designview::designview(BRect rect, char *name, uint32 resizingMode, uint32 flags)
			: BView(rect, name, resizingMode,flags)
{
	//settings
	IsDragging = false;
	ShowGrid = true;
	ShowHandles = true;
	scale = 10;
}

designview::~designview()
{
}

placeholder::placeholder(BRect prect, const char *name, const char *ptype, int32 pid, uint32 resizingMode, uint32 flags)
			: BView(prect, name, resizingMode, flags)
{
	id = pid;
	type = ptype;
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	moving = false;
}

void designview::Draw(BRect)
{
	//grid
	if(ShowGrid == true)
	{
		SetHighColor(0,0,0,0);
		

		int width = (int)Frame().Width()-5;
		int height = (int)Frame().Height()-5;
		int x = scale;
		int y = scale;
			
	
		for(x; x < width; x+=scale)
		{
			for(y; y < height; y+=scale)
			{
				StrokeRect(BRect(x,y,x,y));		
			}
			y=scale;
			
		}
	}	
}


void designview::StartDragging()
{

		IsDragging = true;

		if(!wdwLoc || wdwLoc->IsHidden())
		{
			wdwLoc = new BWindow(BRect(-60,-16,0,0), "wbwLoc", B_BORDERED_WINDOW_LOOK, B_FLOATING_ALL_WINDOW_FEEL, B_WILL_DRAW);
			strLocation = new BStringView(BRect(1,1,59,15), "strLocation", "--,--", B_FOLLOW_ALL, B_WILL_DRAW);
			strLocation->SetAlignment(B_ALIGN_CENTER);
			wdwLoc->AddChild(strLocation);
		}	
		wdwLoc->Show();
}

void designview::StopDragging()
{
	IsDragging = false;
	wdwLoc->Hide();
}

void designview::AddElement(BPoint where)
{
//		IsDragging = false;


		if(strcmp(type, "BOX")==0)
		{
			BBox* box = new BBox(BRect(0,0,200,100), "box", B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE);
			box->SetLabel("Box Title");
			boxes += 1;
			BString name;
			name << "boxBox" << boxes;
			BView* phbox = new placeholder(BRect(where.x,where.y,where.x+206,where.y+101), name.String(), "BBox", CountChildren()+1, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
			AddChild(phbox);
			phbox->AddChild(box);
		}
		else if(strcmp(type, "BUTTON")==0)
		{
			BButton* button = new BButton(BRect(0,0,60,20), "button", "Button", new BMessage(), B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE);
			buttons += 1;
			BString name;
			name << "btnButton" << buttons;
			BView* phbutton = new placeholder(BRect(where.x,where.y,where.x+66,where.y+21), name.String(), "BButton", CountChildren()+1, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
			AddChild(phbutton);
			phbutton->AddChild(button);
		}
		else if(strcmp(type, "PICTUREBUTTON")==0)
		{
			BButton* picturebutton = new BButton(BRect(0,0,80,20), "picturebutton", "Picture Button", new BMessage(), B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE);
			picturebuttons += 1;
			BString name;
			name << "pbnPictureButton" << picturebuttons;
			BView* phpicturebutton = new placeholder(BRect(where.x,where.y,where.x+86,where.y+21), name.String(), "BPictureButton", CountChildren()+1, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
			AddChild(phpicturebutton);
			phpicturebutton->AddChild(picturebutton);
		}
		else if(strcmp(type, "RADIOBUTTON")==0)
		{
			BRadioButton* radiobutton = new BRadioButton(BRect(0,0,100,16), "radiobutton", "Radio Button", new BMessage(), B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE);
			radiobuttons += 1;
			BString name;
			name << "radRadioButton" << radiobuttons;
			BView* phradiobutton = new placeholder(BRect(where.x,where.y,where.x+106,where.y+17), name.String(), "BRadioButton", CountChildren()+1, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
			AddChild(phradiobutton);
			phradiobutton->AddChild(radiobutton);
		}
		else if(strcmp(type, "CHECKBOX")==0)
		{
			BCheckBox* checkbox = new BCheckBox(BRect(0,0,100,16), "checkbox", "Checkbox", new BMessage(), B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE);
			checkboxes += 1;
			BString name;
			name << "chkCheckBox" << checkboxes;
			BView* phcheckbox = new placeholder(BRect(where.x,where.y,where.x+106,where.y+17), name.String(), "BCheckBox", CountChildren()+1, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
			AddChild(phcheckbox);
			phcheckbox->AddChild(checkbox);
		}
		else if(strcmp(type, "STRINGVIEW")==0)
		{
			BStringView* stringview = new BStringView(BRect(0,0,80,16), "stringview", "String View", B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE);
			stringviews += 1;
			BString name;
			name << "strStringView" << stringviews;
			BView* phstringview = new placeholder(BRect(where.x,where.y,where.x+86,where.y+17), name.String(), "BStringView", CountChildren()+1, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
			AddChild(phstringview);
			phstringview->AddChild(stringview);
		}
		else if(strcmp(type, "TEXTCONTROL")==0)
		{
			BTextControl* textcontrol = new BTextControl(BRect(0,0,200,16), "textcontrol", "Text Control", "text", new BMessage(), B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE);
			textcontrols += 1;
			BString name;
			name << "txtTextControl" << textcontrols;
			BView* phtextcontrol = new placeholder(BRect(where.x,where.y,where.x+206,where.y+17), name.String(), "BTextControl", CountChildren()+1, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
			AddChild(phtextcontrol);
			phtextcontrol->AddChild(textcontrol);
		}
		else if(strcmp(type, "TEXTVIEW")==0)
		{
			BTextView* textview = new BTextView(BRect(0,0,250,150), "textview", BRect(where.x,where.y,where.x+200,where.y+400), B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE);
			textviews += 1;
			BString name;
			name << "txtTextView" << textviews;
			BView* phtextview = new placeholder(BRect(where.x,where.y,where.x+256,where.y+151), name.String(), "BTextView", CountChildren()+1, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
			AddChild(phtextview);
			phtextview->AddChild(textview);
		}
			
		Window()->PostMessage(V_ACTION_BUILD_LIST);		
}

void designview::MouseMoved(BPoint point, uint32 transit, const BMessage *message)
{
		BString string;

		if(IsDragging)
		{
				//move our little locationwindow
				ConvertToScreen(&point);
				point.x += 18;
				wdwLoc->MoveTo(point);
				
				//and update text
				ConvertFromScreen(&point);
				point.x -= 18;
				string << "drop a " << type << " at: " << (int)point.x << "," << (int)point.y;
				string.ToLower();
				string.Capitalize();

				wdwLoc->Lock();
				wdwLoc->ResizeTo(be_plain_font->StringWidth(string.String())+10, wdwLoc->Frame().Height());
				strLocation->SetText(string.String());
				wdwLoc->Unlock();
		}
}

void designview::MouseDown(BPoint where)
{
	if(IsDragging)
	{
		AddElement(where);
	}	
}

void designview::ResetCounters()
{
	boxes = buttons = picturebuttons = radiobuttons = checkboxes = stringviews = textcontrols = textviews = 0;
}

void placeholder::Draw(BRect)
{
	if(  ((designview *)Parent())->ShowHandles == true)
	{
	
	//box1
	BRect drect(Bounds().right-4,Bounds().bottom-4,Bounds().right,Bounds().bottom);
	SetHighColor(250,0,0,0);
	StrokeRect(drect);
	SetHighColor(255,255,255,0);
	FillRect(BRect(drect.left+1,drect.top+1,drect.right-1,drect.bottom-1));
	SetHighColor(181,181,181,0);
	FillRect(BRect(drect.left+3,drect.top+3,drect.right-1,drect.bottom-1));

	//box2
	drect.OffsetBy(0,-6);
	SetHighColor(250,0,0,0);
	StrokeRect(drect);
	SetHighColor(255,255,255,0);
	FillRect(BRect(drect.left+1,drect.top+1,drect.right-1,drect.bottom-1));
	SetHighColor(181,181,181,0);
	FillRect(BRect(drect.left+2,drect.top+2,drect.right-2,drect.bottom-2));

	}

}

void placeholder::MouseDown(BPoint down)
{
	moving=true;
	clickPoint = down;

	float border = Bounds().bottom-5;
	if(down.y < border)
	{
		resize = false;
	}
	else
	{
		resize = true;
	}
	SetMouseEventMask(B_POINTER_EVENTS,B_NO_POINTER_HISTORY);
	initialSize = Bounds();  											//get element size
	((designview *)Parent())->StopDragging();		//hide location window

}

void placeholder::MouseUp(BPoint up)
{
	moving=false;
	Invalidate();		//redraw the handles
}

void placeholder::MouseMoved(BPoint move, uint32 transit, const BMessage *message)
{
	if(moving)  //are we draggin'?
	{
		if(resize)  //resizing:
		{
			ResizeTo(initialSize.Width()+(move.x-clickPoint.x), initialSize.Height()+(move.y-clickPoint.y));
		}
		else  //moving:
		{
			MoveBy(move.x-initialSize.Width(), move.y-(initialSize.Height()-5));
		}

	}

}

