
/*//-----------------------------------------------------------------------------
	Niue Development Environment
	Copyright 2008 T. Lansbergen, All Rights Reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted for non-commercial use only.
	
	This software is provided ``as is'' and any express or implied warranties,
	including, but not limited to, the implied warranties of merchantability and
	fitness for a particular purpose are disclaimed. In no event shall
	authors be liable for any direct, indirect, incidental, special,
	exemplary, or consequential damages (including, but not limited to,
	procurement of substitute goods or services; loss of use, data, or profits;
	or business interruption) however caused and on any theory of liability,
	whether in contract, strict liability, or tort (including negligence or
	otherwise) arising in any way out of the use of this software, even if
	advised of the possibility of such damage. 

*///-----------------------------------------------------------------------------


#include "visualwindow.h"

BRect vrect(120,120,900,700);

visualwindow::visualwindow() :
	BWindow(vrect, "Niue Visual Designer", B_DOCUMENT_WINDOW,B_WILL_DRAW)
{

	SetSizeLimits(350, 30000, 150, 30000);
	
	// define objects
	
	//metrics
	int mleft = 150;
	int mright = (int)vrect.Width()-150;
	int mhor = 350;	

	//movers
	mvrLeft = new Mover(BRect(mleft, 0, mleft+6, vrect.Height()), B_FOLLOW_LEFT | B_FOLLOW_TOP_BOTTOM, true);
	mvrRight = new Mover(BRect(mright, 0, mright+6, vrect.Height()), B_FOLLOW_RIGHT | B_FOLLOW_TOP_BOTTOM, true);
	mvrHorz = new Mover(BRect(-1,mhor,mleft-1,mhor+4), B_FOLLOW_LEFT | B_FOLLOW_BOTTOM, false);

	vwItems = new vitemsview(BRect(0,0,mleft-1,mhor-1), "vwItems", B_FOLLOW_LEFT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW);
	vwProps = new vpropsview(BRect(0,mhor+5,mleft-1,vrect.Height()), "vwProps", B_FOLLOW_LEFT | B_FOLLOW_BOTTOM, B_WILL_DRAW | B_FRAME_EVENTS | B_FULL_UPDATE_ON_RESIZE);
	vwDesign = new vdesignview(BRect(mleft+7,0,mright-1,vrect.Height()), "vwDesign", B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW | B_FRAME_EVENTS);
	vwElements = new velementsview(BRect(mright+7,0,vrect.Width(),vrect.Height()), "vwElements", B_FOLLOW_RIGHT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW | B_FRAME_EVENTS);
		
	// set objects
	vwItems->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	vwProps->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	vwDesign->SetViewColor(212,18,24,0);
	vwElements->SetViewColor(150,120,170,0);
	
	// add objects
	AddChild(mvrLeft);
	AddChild(mvrRight);
	AddChild(mvrHorz);

	AddChild(vwItems);
	AddChild(vwProps);
	AddChild(vwDesign);
	AddChild(vwElements);

}


codewindow::codewindow(BString text) :
	BWindow(BRect(0,0,640,400), "Niue Visual Designer - Generated source", B_FLOATING_WINDOW, B_WILL_DRAW)
{
	//center me
	BScreen mScreen(B_MAIN_SCREEN_ID);
	BRect screenRes, winBounds;
	screenRes.Set((mScreen.Frame().left), (mScreen.Frame().top), (mScreen.Frame().right), (mScreen.Frame().bottom));
	winBounds = Bounds();
	MoveBy((screenRes.right / 2) - (winBounds.right / 2), (screenRes.bottom / 2) - (winBounds.bottom / 2));

	txtView = new BTextView(BRect(0,0,winBounds.Width()-B_V_SCROLL_BAR_WIDTH,winBounds.Height()-B_H_SCROLL_BAR_HEIGHT), "txtView", BRect(2,2,30000,30000), B_FOLLOW_ALL_SIDES, B_WILL_DRAW);
	scrView = new BScrollView("scrView", txtView, B_FOLLOW_ALL_SIDES, B_WILL_DRAW, true, true, B_NO_BORDER);
	
	txtView->MakeEditable(false);
	txtView->SetFontAndColor(be_plain_font, B_FONT_ALL, &darkblue);
	txtView->SetText(text.String());

	AddChild(scrView);
}

void visualwindow::BuildList()
{
	//clear all
	vwItems->lstItems->MakeEmpty();
	
	//add all
	for(int i = 0; i  < vwDesign->vwDes->CountChildren(); i++)
	{
		BView* control = vwDesign->vwDes->ChildAt(i);
		vwItems->lstItems->AddItem(new BStringItem(control->Name()));
	}			

}

BString visualwindow::GetContructors()
{
		BString codestring;
		codestring << "\n";
		
		//create a constructor for each placeholder/element
		for(int i = 0; i  < vwDesign->vwDes->CountChildren(); i++)
		{
			placeholder* control = dynamic_cast<placeholder*>(vwDesign->vwDes->ChildAt(i));
			codestring << "\t" << control->Name() << " = new " << control->type << "\n";
		}

		return codestring;	

}

bool visualwindow::QuitRequested()
{
	Close();
}

void visualwindow::MessageReceived(BMessage* message)
{
	
	switch(message->what)
	{
		case MOVER_MESSAGE:
		{
			Mover *mv;
			if (message->FindPointer(MM_VIEW,(void**)&mv) == B_OK )
			{
				float coord = message->FindFloat(MM_COORD);


				if (mv == mvrLeft)
				{
					BPoint p = ConvertFromScreen(BPoint(coord,0));

					vwItems->ResizeTo(p.x, vwItems->Frame().Height());
					vwProps->ResizeTo(p.x, vwProps->Frame().Height());

					vwDesign->ResizeTo(Frame().Width()-p.x-mvrLeft->Frame().Width()-mvrRight->Frame().Width()-vwElements->Frame().Width()-3, vwDesign->Frame().Height());
					vwDesign->MoveTo(p.x+mvrLeft->Frame().Width()+1 , vwDesign->Frame().top);					

					mvrHorz->ResizeTo(p.x, mvrHorz->Frame().Height());
					mvrLeft->MoveTo(p.x, mvrLeft->Frame().top);		
			
				}
				else if (mv == mvrRight)
				{
					BPoint p = ConvertFromScreen(BPoint(coord,0));

					vwDesign->ResizeTo(p.x-vwItems->Frame().Width()-mvrLeft->Frame().Width()-3, vwDesign->Frame().Height());
					
					vwElements->ResizeTo(Frame().Width()-p.x-mvrRight->Frame().Width(), vwElements->Frame().Height());
					vwElements->MoveTo(p.x+mvrRight->Frame().Width(), vwElements->Frame().top); 

					mvrRight->MoveTo(p.x, mvrRight->Frame().top);
				}
/*
				else if (mv == mvrHorz)
				{
					BPoint p = ConvertFromScreen(BPoint(0,coord));

					vwItems->ResizeTo(vwItems->Frame().Width(), p.y);

					vwProps->ResizeTo(vwProps->Frame().Width(), Frame().Height()-p.y-mvrHorz->Frame().Height());
					vwProps->MoveTo(vwProps->Frame().left, p.y+mvrHorz->Frame().Height());
						
					mvrHorz->MoveTo(mvrHorz->Frame().left, p.y);
				}
*/
			}
		}
		break;

		case V_ACTION_CREATE_CODE:
		{
				//stop dragging
				if(vwDesign->vwDes->IsDragging)
				{
				vwDesign->vwDes->StopDragging();
				}

				BWindow* wdwCode = new codewindow(GetContructors());
				wdwCode->Show();
		}
		break;

		case V_ACTION_BUILD_LIST:
		{
			BuildList();
		}
		break;

		case V_OPTION_SHOW_GRID:
		{
				//stop dragging
				if(vwDesign->vwDes->IsDragging)
				{
				vwDesign->vwDes->StopDragging();
				}

				if(vwElements->chkGrid->Value()==1)
				{
					vwDesign->vwDes->ShowGrid = true;
				}
				else if(vwElements->chkGrid->Value()==0)
				{
					vwDesign->vwDes->ShowGrid = false;
				}
				vwDesign->vwDes->Invalidate();	
		}
		break;

		case V_OPTION_SHOW_HANDLES:
		{
				//stop dragging
				if(vwDesign->vwDes->IsDragging)
				{
				vwDesign->vwDes->StopDragging();
				}

				if(vwElements->chkHandles->Value()==1)
				{
					vwDesign->vwDes->ShowHandles = true;
				}
				else if(vwElements->chkHandles->Value()==0)
				{
					vwDesign->vwDes->ShowHandles = false;
				}
//				vwDesign->vwDes->Invalidate();

				//auch, need to invalidate all our items	
				for(int i = 0; i  < vwDesign->vwDes->CountChildren(); i++)
				{
					vwDesign->vwDes->ChildAt(i)->Invalidate();
				}			

		}
		break;

		case V_OPTION_ALIGN_VERT:
		{
				//stop dragging
				if(vwDesign->vwDes->IsDragging)
				{
				vwDesign->vwDes->StopDragging();
				}

				int32 elements = vwDesign->vwDes->CountChildren();
				int r = vwDesign->vwDes->scale*2;

				for(int i=0; i < elements; i++)
				{
					BView* control = vwDesign->vwDes->ChildAt(i);

					//do some rounding
					int y = (int)control->Frame().top;
					int ry = y%r;
	
					if(ry <= r/2)  //round down
					{
							y-=ry;
					}
					else	//round up
					{
							y+=r-ry;
					}
					
					control->MoveTo(control->Frame().left, y);
				}

		}
		break;

		case V_OPTION_ALIGN_HORZ:
		{
				//stop dragging
				if(vwDesign->vwDes->IsDragging)
				{
				vwDesign->vwDes->StopDragging();
				}

				int32 elements = vwDesign->vwDes->CountChildren();
				int r = vwDesign->vwDes->scale*2;

				for(int i=0; i < elements; i++)
				{
					BView* control = vwDesign->vwDes->ChildAt(i);

					//do some rounding
					int x = (int)control->Frame().left;
					int rx = x%r;

					if(rx <= r/2)	//round down
					{
							x-=rx;
					}
					else	//round up
					{
						x+=r-rx;
					}					
	
					control->MoveTo(x, control->Frame().top);
				}

		}
		break;

		case V_OPTION_KILL_ALL:
		{
				//stop dragging
				if(vwDesign->vwDes->IsDragging)
				{
				vwDesign->vwDes->StopDragging();
				}

				//remove elements from grid
				while(vwDesign->vwDes->CountChildren() != 0)
				{
						vwDesign->vwDes->ChildAt(0)->RemoveSelf();
				}

				//reset counters
				vwDesign->vwDes->ResetCounters();

				//update the lsit
				BuildList();

		}
		break;

		case V_ADD_BOX_MSG:
		{
				vwDesign->vwDes->type = "BOX";
				vwDesign->vwDes->StartDragging();
		}
		break;
		case V_ADD_BUTTON_MSG:
		{
				vwDesign->vwDes->type = "BUTTON";
				vwDesign->vwDes->StartDragging();
		}	
		break;
		case V_ADD_PICTUREBUTTON_MSG:
		{
				vwDesign->vwDes->type = "PICTUREBUTTON";
				vwDesign->vwDes->StartDragging();
		}	
		break;
		case V_ADD_RADIOBUTTON_MSG:
		{
				vwDesign->vwDes->type = "RADIOBUTTON";
				vwDesign->vwDes->StartDragging();
		}	
		break;
		case V_ADD_CHECKBOX_MSG:
		{
				vwDesign->vwDes->type = "CHECKBOX";
				vwDesign->vwDes->StartDragging();
		}	
		break;
		case V_ADD_STRINGVIEW_MSG:
		{
				vwDesign->vwDes->type = "STRINGVIEW";
				vwDesign->vwDes->StartDragging();
		}	
		break;
		case V_ADD_TEXTCONTROL_MSG:
		{
				vwDesign->vwDes->type = "TEXTCONTROL";
				vwDesign->vwDes->StartDragging();
		}	
		break;
		case V_ADD_TEXTVIEW_MSG:
		{
				vwDesign->vwDes->type = "TEXTVIEW";
				vwDesign->vwDes->StartDragging();
		}	
		break;
	}
}


